/* Terraform - (C) 1997_1999 Robert Gasch (Robert_Gasch@peoplesoft.com)
 *  - www.peoplesoft.com/peoplepages/g/robert_gasch/terraform/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111_1307, USA.
 */


#include "TFDialogOptions.h"
#include "TFOptions.h"
#include "GlobalTrace.h"
#include "GlobalSanityCheck.h"


/*
 *  constructor: initialize all data members 
 */
TFDialogOptions::TFDialogOptions (HeightField *HF, HeightFieldDraw *HFD)
	 : GuiDialogOAC ("Terraform Options Dialog", NULL),
	   d_frame1 ("Global Draw Options"), 
	   d_frame2("Height Field Draw Options"),
	   d_frame3("Fast Wireframe Options"),
	   d_frame4("POV Render Options")
{
	GlobalTrace::trace (GlobalTrace::TRACE_FLOW, "+++ TFDialogOptions\n");

	d_vBox1 = new Gtk_VBox (TRUE, 2);
	d_vBox2 = new Gtk_VBox (TRUE, 2);
	d_vBox3 = new Gtk_VBox (TRUE, 2);
	d_vBox4 = new Gtk_VBox (TRUE, 2);
	d_hBox1_1 = new Gtk_HBox (TRUE, 5);
	d_hBox1_2 = new Gtk_HBox (TRUE, 5);
	d_hBox1_3 = new Gtk_HBox (TRUE, 5);
	d_hBox1_4 = new Gtk_HBox (TRUE, 5);
	d_hBox2_1 = new Gtk_HBox (TRUE, 5);
	d_hBox2_2 = new Gtk_HBox (TRUE, 5);
	d_hBox2_3 = new Gtk_HBox (TRUE, 5);
	d_hBox3_1 = new Gtk_HBox (TRUE, 5);
	d_hBox3_2 = new Gtk_HBox (TRUE, 5);
	d_hBox3_3 = new Gtk_HBox (TRUE, 5);
	d_hBox4_1 = new Gtk_HBox (TRUE, 5);
	d_hBox4_2 = new Gtk_HBox (TRUE, 5);
	d_hBox4_3 = new Gtk_HBox (TRUE, 5);
	d_hBox4_4 = new Gtk_HBox (TRUE, 5);

	d_lblTitle1 = new Gtk_Label ("Global");
	d_lblTitle2 = new Gtk_Label ("Height Field View");
	d_lblTitle3 = new Gtk_Label ("Fast Wireframe");
	d_lblTitle4 = new Gtk_Label ("PovRay");
	d_lblScale = new Gtk_Label ("Display Scale");
	d_lblSync = new Gtk_Label ("Sync Rate");
	d_lblResolution = new Gtk_Label ("Wireframe Resolution");
	d_lblYScale = new Gtk_Label ("Y Scale Factor");
	d_lblSealevel = new Gtk_Label ("Sealevel");
	d_lblFillSea = new Gtk_Label ("Sea is Filled");
	d_lblDrawDerived = new Gtk_Label ("Draw derived data");
	d_lblFastResolution = new Gtk_Label ("Half Wireframe Resolution");
	d_lblFastYscale = new Gtk_Label ("Half Y Scale Factor");
	d_lblFastZrot = new Gtk_Label ("Enable Z-axis rotation");
	d_lblPOVfileName = new Gtk_Label ("POV template file");
	d_lblPOVwidth = new Gtk_Label ("Render width");
	d_lblPOVyscale = new Gtk_Label ("Half Y scale");
	d_lblPOVkeepFiles = new Gtk_Label ("Keep generated files?");

	d_adjScale = new Gtk_Adjustment (1.0, 1.0, 3.0, 1.0);
	d_adjSync = new Gtk_Adjustment (1, 1, 5000, 1);
	d_adjResolution = new Gtk_Adjustment (5.0, 1.0, 10.0, 1.0);
	d_adjYScale = new Gtk_Adjustment (0.33, 0, 1, 0.01, 10, 0);
	d_adjSealevel = new Gtk_Adjustment (0.33, 0, 1, 0.01, 10, 0);
	d_adjPOVwidth = new Gtk_Adjustment (400, 100, 1000, 1, 10, 0);

	d_hsScale = new Gtk_HScale (*d_adjScale);
	d_hsSync = new Gtk_HScale (*d_adjSync);
	d_hsResolution = new Gtk_HScale (*d_adjResolution);
	d_hsYScale = new Gtk_HScale (*d_adjYScale);
	d_hsSealevel = new Gtk_HScale (*d_adjSealevel);
	d_hsPOVwidth = new Gtk_HScale (*d_adjPOVwidth);

	setHFobjs (HF, HFD);

	buildDialogWindow ();
	updateSliders ();
}


/*
 *  destructor: clean up
 */
TFDialogOptions::~TFDialogOptions ()
{
	delete d_vBox1;
	delete d_vBox2;
	delete d_vBox3;
	delete d_vBox4;
	delete d_hBox1_1;
	delete d_hBox1_2;
	delete d_hBox1_3;
	delete d_hBox1_4;
	delete d_hBox2_1;
	delete d_hBox2_2;
	delete d_hBox2_3;
	delete d_hBox3_1;
	delete d_hBox3_2;
	delete d_hBox3_3;
	delete d_hBox4_1;
	delete d_hBox4_2;
	delete d_hBox4_3;
	delete d_hBox4_4;

	delete d_lblTitle1;
	delete d_lblTitle2;
	delete d_lblTitle3;
	delete d_lblTitle4;
	delete d_lblScale;
	delete d_lblSync;
	delete d_lblResolution;
	delete d_lblYScale;
	delete d_lblSealevel;
	delete d_lblFillSea;
	delete d_lblDrawDerived;
	delete d_lblFastResolution;
	delete d_lblFastYscale;
	delete d_lblFastZrot;
	delete d_lblPOVfileName;
	delete d_lblPOVwidth;
	delete d_lblPOVyscale;
	delete d_lblPOVkeepFiles;

	delete d_adjScale;
	delete d_adjSync;
	delete d_adjResolution;
	delete d_adjYScale;
	delete d_adjSealevel;
	delete d_adjPOVwidth;

	delete d_hsScale;
	delete d_hsSync;
	delete d_hsResolution;
	delete d_hsYScale;
	delete d_hsSealevel;
	delete d_hsPOVwidth;

	GlobalTrace::trace (GlobalTrace::TRACE_FLOW, "--- TFDialogOptions\n");
}


/*
 *  setHFobjs: set the HF obejcts we operate on 
 */
void TFDialogOptions::setHFobjs (HeightField *HF, HeightFieldDraw *HFD)
{
	SanityCheck::bailout ((!HF), "HF==NULL", "TFDialogOptions::setHFobjs");
	SanityCheck::bailout ((!HFD), "HFD==NULL", "TFDialogOptions::setHFobjs");

	d_HF = HF;
	d_HFD = HFD;
}


/*
 *  updateSliders: update the sliders with new/changed values from TFOptions
 */
void TFDialogOptions::updateSliders ()
{
	d_cbFillSea.set_active (TFOptions::s_fillSea);
	d_adjScale->set_value (TFOptions::s_scale); 
	d_adjSync->set_value (TFOptions::s_syncRate);
	d_adjResolution->set_value (TFOptions::s_resolution); 
	d_adjYScale->set_value (TFOptions::s_yscale); 
	d_adjSealevel->set_value (TFOptions::s_sealevel); 
	d_cbFastResolution.set_active (TFOptions::s_fastHalfRes);
	d_cbFastYscale.set_active (TFOptions::s_fastHalfYscale);
	d_cbFastZrot.set_active (TFOptions::s_fastZrot);
}


/*
 *  fillVBox: fill the dialog's VBox
 */
void TFDialogOptions::fillVBox ()
{
	this->get_vbox()->pack_start (d_notebook, TRUE, TRUE, 0);
	d_frame1.set_shadow_type (GTK_SHADOW_ETCHED_IN);
	d_frame2.set_shadow_type (GTK_SHADOW_ETCHED_IN);
	d_frame3.set_shadow_type (GTK_SHADOW_ETCHED_IN);
	d_frame4.set_shadow_type (GTK_SHADOW_ETCHED_IN);
	d_notebook.append_page (d_frame1, *d_lblTitle1);
	d_notebook.append_page (d_frame2, *d_lblTitle2);
	d_notebook.append_page (d_frame3, *d_lblTitle3);
	d_notebook.append_page (d_frame4, *d_lblTitle4);
	d_notebook.show ();

	buildDrawOptionsFrame ();
	buildHFDrawOptionsFrame ();
	buildFastWireOptionsFrame ();
	buildPOVOptionsFrame ();
}


/*
 *  buildDialogWindow:
 */
void TFDialogOptions::buildDialogWindow ()
{
	this->set_usize (350, 250);
	this->set_title (d_winTitle);

	this->get_vbox()->set_border_width (2);

	this->setupVBox ();
	fillVBox ();
	fillActionArea ();

	this->show ();
}


/*
 *  buildDrawOptionsFrame: frame for the global draw options
 */
void TFDialogOptions::buildDrawOptionsFrame ()
{
	d_lblScale->set_justify (GTK_JUSTIFY_LEFT);
	d_hBox1_1->pack_start (*d_lblScale, TRUE, TRUE, s_HBOff);
	d_lblScale->show ();
        d_hsScale->set_update_policy (GTK_UPDATE_DELAYED);
        d_hsScale->set_digits (0);
        d_hsScale->set_draw_value (TRUE);
	d_hBox1_1->pack_start (*d_hsScale, TRUE, TRUE, s_HBOff);
        d_hsScale->show ();
	d_vBox1->pack_start (*d_hBox1_1, TRUE, TRUE, s_VBOff);
	d_hBox1_1->show ();

	d_lblSync->set_justify (GTK_JUSTIFY_LEFT);
	d_hBox1_2->pack_start (*d_lblSync, TRUE, TRUE, s_HBOff);
	d_lblSync->show ();
        d_hsSync->set_update_policy (GTK_UPDATE_DELAYED);
        d_hsSync->set_digits (0);
        d_hsSync->set_draw_value (TRUE);
	d_hBox1_2->pack_start (*d_hsSync, TRUE, TRUE, s_HBOff);
        d_hsSync->show ();
	d_vBox1->pack_start (*d_hBox1_2, TRUE, TRUE, s_VBOff);
	d_hBox1_2->show ();

	d_lblFillSea->set_justify (GTK_JUSTIFY_LEFT);
	d_hBox1_3->pack_start (*d_lblFillSea, TRUE, TRUE, s_HBOff);
	d_lblFillSea->show ();
	d_cbFillSea.set_active (TRUE); 
	d_hBox1_3->pack_start (d_cbFillSea, TRUE, TRUE, s_HBOff);
	d_cbFillSea.show ();
	d_vBox1->pack_start (*d_hBox1_3, TRUE, TRUE, s_VBOff);
	d_hBox1_3->show ();

	d_lblDrawDerived->set_justify (GTK_JUSTIFY_LEFT);
	d_hBox1_4->pack_start (*d_lblDrawDerived, TRUE, TRUE, s_HBOff);
	d_lblDrawDerived->show ();
	d_cbDrawDerived.set_active (TRUE); 
	d_hBox1_4->pack_start (d_cbDrawDerived, TRUE, TRUE, s_HBOff);
	d_cbDrawDerived.show ();
	d_vBox1->pack_start (*d_hBox1_4, TRUE, TRUE, s_VBOff);
	d_hBox1_4->show ();


	d_frame1.add (d_vBox1);
	d_vBox1->show ();
	d_frame1.show ();
}


/*
 *  buildHFDrawOptionsFrame: frame for the Height Field draw options
 */
void TFDialogOptions::buildHFDrawOptionsFrame ()
{
	d_lblResolution->set_justify (GTK_JUSTIFY_LEFT);
	d_hBox2_1->pack_start (*d_lblResolution, TRUE, TRUE, s_HBOff);
	d_lblResolution->show ();
        d_hsResolution->set_update_policy (GTK_UPDATE_DELAYED);
        d_hsResolution->set_digits (0);
        d_hsResolution->set_draw_value (TRUE);
	d_hBox2_1->pack_start (*d_hsResolution, TRUE, TRUE, s_HBOff);
        d_hsResolution->show ();
	d_vBox2->pack_start (*d_hBox2_1, TRUE, TRUE, s_VBOff);
	d_hBox2_1->show ();

	d_lblYScale->set_justify (GTK_JUSTIFY_LEFT);
	d_hBox2_2->pack_start (*d_lblYScale, TRUE, TRUE, s_HBOff);
	d_lblYScale->show ();
        d_hsYScale->set_update_policy (GTK_UPDATE_DELAYED);
        d_hsYScale->set_digits (2);
        d_hsYScale->set_draw_value (TRUE);
	d_hBox2_2->pack_start (*d_hsYScale, TRUE, TRUE, s_HBOff);
        d_hsYScale->show ();
	d_vBox2->pack_start (*d_hBox2_2, TRUE, TRUE, s_VBOff);
	d_hBox2_2->show ();

	d_lblSealevel->set_justify (GTK_JUSTIFY_LEFT);
	d_hBox2_3->pack_start (*d_lblSealevel, TRUE, TRUE, s_HBOff);
	d_lblSealevel->show ();
        d_hsSealevel->set_update_policy (GTK_UPDATE_DELAYED);
        d_hsSealevel->set_digits (2);
        d_hsSealevel->set_draw_value (TRUE);
	d_hBox2_3->pack_start (*d_hsSealevel, TRUE, TRUE, s_HBOff);
        d_hsSealevel->show ();
	d_vBox2->pack_start (*d_hBox2_3, TRUE, TRUE, s_VBOff);
	d_hBox2_3->show ();

	d_frame2.add (d_vBox2);
	d_vBox2->show ();
	d_frame2.show ();
}

/*
 *  buildFastDrawOptionsFrame: frame for the Height Field draw options
 */
void TFDialogOptions::buildFastWireOptionsFrame ()
{
	d_lblFastResolution->set_justify (GTK_JUSTIFY_LEFT);
	d_hBox3_1->pack_start (*d_lblFastResolution, TRUE, TRUE, s_HBOff);
	d_lblFastResolution->show ();
	d_cbFastResolution.set_active (TRUE);
	d_hBox3_1->pack_start (d_cbFastResolution, TRUE, TRUE, s_HBOff);
	d_cbFastResolution.show ();
	d_vBox3->pack_start (*d_hBox3_1, TRUE, TRUE, s_VBOff);
	d_hBox3_1->show ();

	d_lblFastYscale->set_justify (GTK_JUSTIFY_LEFT);
	d_hBox3_2->pack_start (*d_lblFastYscale, TRUE, TRUE, s_HBOff);
	d_lblFastYscale->show ();
	d_cbFastYscale.set_active (TRUE);
	d_hBox3_2->pack_start (d_cbFastYscale, TRUE, TRUE, s_HBOff);
	d_cbFastYscale.show ();
	d_vBox3->pack_start (*d_hBox3_2, TRUE, TRUE, s_VBOff);
	d_hBox3_2->show ();

	d_lblFastZrot->set_justify (GTK_JUSTIFY_LEFT);
	d_hBox3_3->pack_start (*d_lblFastZrot, TRUE, TRUE, s_HBOff);
	d_lblFastZrot->show ();
	d_cbFastZrot.set_active (FALSE);
	d_hBox3_3->pack_start (d_cbFastZrot, TRUE, TRUE, s_HBOff);
	d_cbFastZrot.show ();
	d_vBox3->pack_start (*d_hBox3_3, TRUE, TRUE, s_VBOff);
	d_hBox3_3->show ();

	d_frame3.add (d_vBox3);
	d_vBox3->show ();
	d_frame3.show ();

}


/*
 *  buildPOVOptionsFrame: build the frame for the Export->POV menu options
 */
void TFDialogOptions::buildPOVOptionsFrame ()
{
	d_lblPOVfileName->set_justify (GTK_JUSTIFY_LEFT);
	d_hBox4_1->pack_start (*d_lblPOVfileName, TRUE, TRUE, s_HBOff);
	d_lblPOVfileName->show ();
	d_enPOVfile.set_text (TFOptions::s_POVfile);
	d_hBox4_1->pack_start (d_enPOVfile, TRUE, TRUE, s_HBOff);
	d_enPOVfile.show ();
	d_vBox4->pack_start (*d_hBox4_1, TRUE, TRUE, s_VBOff);
	d_hBox4_1->show ();

	d_lblPOVwidth->set_justify (GTK_JUSTIFY_LEFT);
	d_hBox4_2->pack_start (*d_lblPOVwidth, TRUE, TRUE, s_HBOff);
	d_lblPOVwidth->show ();
	d_adjPOVwidth->set_value (d_HF->getWidth());
	d_hBox4_2->pack_start (*d_hsPOVwidth, TRUE, TRUE, s_HBOff);
	d_hsPOVwidth->show ();
        d_hsScale->set_digits (0);
	d_vBox4->pack_start (*d_hBox4_2, TRUE, TRUE, s_VBOff);
	d_hBox4_2->show ();

	d_lblPOVyscale->set_justify (GTK_JUSTIFY_LEFT);
	d_hBox4_3->pack_start (*d_lblPOVyscale, TRUE, TRUE, s_HBOff);
	d_lblPOVyscale->show ();
	d_cbPOVhalfYscale.set_active (TRUE);
	d_hBox4_3->pack_start (d_cbPOVhalfYscale, TRUE, TRUE, s_HBOff);
	d_cbPOVhalfYscale.show ();
	d_vBox4->pack_start (*d_hBox4_3, TRUE, TRUE, s_VBOff);
	d_hBox4_3->show ();

	d_lblPOVkeepFiles->set_justify (GTK_JUSTIFY_LEFT);
	d_hBox4_4->pack_start (*d_lblPOVkeepFiles, TRUE, TRUE, s_HBOff);
	d_lblPOVkeepFiles->show ();
	d_cbPOVkeepFiles.set_active (FALSE);
	d_hBox4_4->pack_start (d_cbPOVkeepFiles, TRUE, TRUE, s_HBOff);
	d_cbPOVkeepFiles.show ();
	d_vBox4->pack_start (*d_hBox4_4, TRUE, TRUE, s_VBOff);
	d_hBox4_4->show ();

	d_frame4.add (d_vBox4);
	d_vBox4->show ();
	d_frame4.show ();

}


/*
 *  getValues: get all the values from the dialog inputs
 */
void TFDialogOptions::getValues (bool *fillSea, bool *drawDerived, int *scale, 
				int *syncRate, int *resolution, float *yscale, 
				float *sealevel, bool *fastHalfRes, 
				bool *fastHalfYscale, bool *fastZrot)
{
	*fillSea = d_cbFillSea.get_active ();
	*drawDerived = d_cbDrawDerived.get_active ();
	*scale = (int)(d_adjScale->get_value ()); 
	*syncRate = (int)(d_adjSync->get_value());
	*resolution = (int)(d_adjResolution->get_value ()); 
	*yscale= d_adjYScale->get_value (); 
	*sealevel = d_adjSealevel->get_value (); 
	*fastHalfRes = d_cbFastResolution.get_active ();
	*fastHalfYscale = d_cbFastYscale.get_active ();
	*fastZrot = d_cbFastZrot.get_active ();
}


void TFDialogOptions::buttonCallbackOK ()
{	
	this->hide ();
	gdk_flush ();
	buttonCallbackApply ();
}


/*
 *  buttonCallbackApply: update what's been changed and redraw if necessary
 */
void TFDialogOptions::buttonCallbackApply ()
{
	bool			redraw=FALSE;

	int			scale, 
				syncRate;
	bool			fillSea,
				drawDerived,
				fastHalfRes,
				fastHalfYscale,
				fastZrot; 

	int			resolution;
	float			yscale, 
				sealevel;

	getValues (&fillSea, &drawDerived, &scale, &syncRate, &resolution, &yscale, &sealevel,
		   &fastHalfRes, &fastHalfYscale, &fastZrot);

	if (fillSea != TFOptions::s_fillSea)
		{
		TFOptions::s_fillSea = fillSea;
		redraw = TRUE;
		}

	if (drawDerived != TFOptions::s_drawDerivedData)
		{
		TFOptions::s_drawDerivedData = drawDerived;
		redraw = TRUE;
		}

	if (scale != TFOptions::s_scale)	
		{
		TFOptions::s_scale = scale;
		redraw = TRUE;
		}

	if (syncRate != TFOptions::s_syncRate)
		{
		TFOptions::s_syncRate = syncRate;
		redraw = TRUE;
		}

	if (resolution != TFOptions::s_resolution)
		{
		TFOptions::s_resolution = resolution;
		redraw = TRUE;
		}

	if (yscale != TFOptions::s_yscale)
		{
		TFOptions::s_yscale = yscale;
		redraw = TRUE;
		}

	if (sealevel != TFOptions::s_sealevel)
		{
		TFOptions::s_sealevel = sealevel;
		redraw = TRUE;
		}

	d_HFD->updateParams ();

	TFOptions::s_fastHalfRes = fastHalfRes;
	TFOptions::s_fastHalfYscale = fastHalfRes;
	TFOptions::s_fastZrot = fastZrot;
	d_HFD->setFastOptions (fastHalfRes, fastHalfYscale, fastZrot);

	strcpy (TFOptions::s_POVfile, d_enPOVfile.get_text().c_str());
	TFOptions::s_POVwidth = (int)(d_adjPOVwidth->get_value());
	TFOptions::s_POVhalfYscale = d_cbPOVhalfYscale.get_active();
	TFOptions::s_POVkeepFiles = d_cbPOVkeepFiles.get_active ();

	if (redraw)
		d_HFD->draw ();
}


gint TFDialogOptions::delete_event_impl (GdkEventAny *e)
{
	this->hide ();
	return (0);
}




